@@ -419,6 +419,25 @@ typedef struct {
 	uint8_t mode_control;
 	uint8_t enable_bits;
 	bool blend;
+
+	// Plus and InColor
+	uint8_t xMode;
+	uint8_t underline;
+	uint8_t strikethrough;
+
+	// InColor
+	uint8_t exception;
+	uint8_t planemask_protect;
+	uint8_t planemask_visible;
+	uint8_t maskpolarity;
+	uint8_t write_mode;
+	uint8_t dont_care;
+	uint8_t bgcolor;
+	uint8_t fgcolor;
+	uint8_t latchprotect;
+	uint8_t palette_index;
+	uint8_t palette[16];
+	uint32_t latch;
 } VGA_HERC;
 
 typedef struct {
