@@ -35,14 +35,16 @@ August 8 2005		cyberwalker
 extern unsigned int lfn_id[256];
 extern bool enable_network_redirector;
 extern uint16_t	NetworkHandleList[127];	/*in dos_files.cpp*/
+extern char * DBCS_upcase(char * str);
+extern bool filename_not_8x3(const char *n), filename_not_strict_8x3(const char *n);
 extern bool CodePageHostToGuestUTF16(char *d/*CROSS_LEN*/,const uint16_t *s/*CROSS_LEN*/);
 extern bool CodePageGuestToHostUTF16(uint16_t *d/*CROSS_LEN*/,const char *s/*CROSS_LEN*/);
 
  bool Network_IsNetworkResource(const char * filename)
 {
-	if(strlen(filename)>1 && enable_network_redirector && !control->SecureMode() && (filename[0]=='\\' && filename[1]=='\\' || strlen(filename)>2 && filename[0]=='"' && filename[1]=='\\' && filename[2]=='\\')) {
+	if(strlen(filename)>1 && enable_network_redirector && !control->SecureMode() && ((filename[0]=='\\' && filename[1]=='\\') || (strlen(filename)>2 && filename[0]=='"' && filename[1]=='\\' && filename[2]=='\\'))) {
         char *p = strrchr_dbcs((char *)filename, '\\');
-        return p && (filename[0]=='\\' && p > filename+1 || filename[0]=='"' && p > filename+2);
+        return p && ((filename[0]=='\\' && p > filename+1) || (filename[0]=='"' && p > filename+2));
     } else
 		return false;
 }//bool	Network_IsNetworkFile(uint16_t entry)
@@ -62,7 +64,7 @@ bool usefdw = false;
 HANDLE hFind = INVALID_HANDLE_VALUE;
  bool Network_FindNext(DOS_DTA & dta)
  {
-	uint8_t fattr;char pattern[CROSS_LEN];
+	uint8_t fattr;char pattern[CROSS_LEN],sfn[DOS_NAMELENGTH_ASCII];
 	uint16_t date, time, attr;
 	std::string name,lname;
 	SYSTEMTIME lt, local;
@@ -78,12 +80,21 @@ HANDLE hFind = INVALID_HANDLE_VALUE;
                     name = fda.cAlternateFileName;
                 else {
                     name = !usefdw ? fda.cFileName : (CodePageHostToGuestUTF16(pattern,(uint16_t *)fdw.cFileName) ? pattern : "");
-                    std::transform(name.begin(), name.end(), name.begin(), ::toupper);
+                    if (name == "." || name == "..")
+                        ;
+                    else if (filename_not_8x3(name.c_str()))
+                        name = "";
+                    else if (filename_not_strict_8x3(name.c_str())) {
+                        strcpy(sfn, name.c_str());
+                        DBCS_upcase(sfn);
+                        name = sfn;
+                    }
+                    if (name.empty()) name = "?";
                 }
                 date = DOS_PackDate(lt.wYear,lt.wMonth,lt.wDay);
                 time = DOS_PackTime(lt.wHour,lt.wMinute,lt.wSecond);
                 attr = ((usefdw?fdw.dwFileAttributes:fda.dwFileAttributes) & FILE_ATTRIBUTE_DIRECTORY ? DOS_ATTR_DIRECTORY : 0) | ((usefdw?fdw.dwFileAttributes:fda.dwFileAttributes) & 0x3f);
-                dta.SetResult(name.c_str(),!usefdw ? fda.cFileName : (CodePageHostToGuestUTF16(pattern,(uint16_t *)fdw.cFileName) ? pattern : name.c_str()),usefdw?fdw.nFileSizeLow:fda.nFileSizeLow,date,time,attr);
+                dta.SetResult(name.c_str(),!usefdw ? fda.cFileName : (CodePageHostToGuestUTF16(pattern,(uint16_t *)fdw.cFileName) ? pattern : name.c_str()),usefdw?fdw.nFileSizeLow:fda.nFileSizeLow,usefdw?fdw.nFileSizeHigh:fda.nFileSizeHigh,date,time,attr);
                 return true;
             }
         }
@@ -98,7 +109,7 @@ HANDLE hFind = INVALID_HANDLE_VALUE;
 
  bool Network_FindFirst(const char * _dir,DOS_DTA & dta)
 {
-	uint8_t fattr;char pattern[CROSS_LEN];
+	uint8_t fattr;char pattern[CROSS_LEN],sfn[DOS_NAMELENGTH_ASCII];
 	uint16_t date, time, attr;
 	std::string name;
 	SYSTEMTIME lt, local;
@@ -123,12 +134,21 @@ HANDLE hFind = INVALID_HANDLE_VALUE;
                     name = fda.cAlternateFileName;
                 else {
                     name = !usefdw ? fda.cFileName : (CodePageHostToGuestUTF16(pattern,(uint16_t *)fdw.cFileName) ? pattern : "");
-                    std::transform(name.begin(), name.end(), name.begin(), ::toupper);
+                    if (name == "." || name == "..")
+                        ;
+                    else if (filename_not_8x3(name.c_str()))
+                        name = "";
+                    else if (filename_not_strict_8x3(name.c_str())) {
+                        strcpy(sfn, name.c_str());
+                        DBCS_upcase(sfn);
+                        name = sfn;
+                    }
+                    if (name.empty()) name = "?";
                 }
                 date = DOS_PackDate(lt.wYear,lt.wMonth,lt.wDay);
                 time = DOS_PackTime(lt.wHour,lt.wMinute,lt.wSecond);
                 attr = ((usefdw?fdw.dwFileAttributes:fda.dwFileAttributes) & FILE_ATTRIBUTE_DIRECTORY ? DOS_ATTR_DIRECTORY : 0) | ((usefdw?fdw.dwFileAttributes:fda.dwFileAttributes) & 0x3f);
-                dta.SetResult(name.c_str(),!usefdw ? fda.cFileName : (CodePageHostToGuestUTF16(pattern,(uint16_t *)fdw.cFileName) ? pattern : name.c_str()),usefdw?fdw.nFileSizeLow:fda.nFileSizeLow,date,time,attr);
+                dta.SetResult(name.c_str(),!usefdw ? fda.cFileName : (CodePageHostToGuestUTF16(pattern,(uint16_t *)fdw.cFileName) ? pattern : name.c_str()),usefdw?fdw.nFileSizeLow:fda.nFileSizeLow,usefdw?fdw.nFileSizeHigh:fda.nFileSizeHigh,date,time,attr);
                 return true;
             }
         } while (usefdw?FindNextFileW(hFind, &fdw):FindNextFile(hFind, &fda));
@@ -396,9 +416,9 @@ extern "C" int _nhandle;
  bool Network_CloseFile(uint16_t entry)
 {
 	uint32_t handle=RealHandle(entry);
-	int _Expr_val=!!((handle >= 0 && (unsigned)handle < (unsigned)_nhandle));
+	int _Expr_val=!!(handle >= 0 && (unsigned)handle < (unsigned)_nhandle);
 	//_ASSERT_EXPR( ( _Expr_val ), _CRT_WIDE(#(handle >= 0 && (unsigned)handle < (unsigned)_nhandle)) );
-	if (!(handle > 0) || ( !( _Expr_val ))) {
+	if (!(handle > 0) || ( ! _Expr_val )) {
 		_doserrno = 0L;
 		errno = EBADF;
 		dos.errorcode=(uint16_t)_doserrno;
