@@ -50,6 +50,7 @@
 #endif
 #include "build_timestamp.h"
 
+extern bool shell_keyboard_flush;
 extern bool dos_shell_running_program, mountwarning, winautorun;
 extern bool startcmd, startwait, startquiet, internal_program;
 extern bool addovl, addipx, addne2k, enableime, showdbcs;
@@ -160,7 +161,7 @@ void AutoexecObject::Install(const std::string &in) {
 
 	//autoexec.bat is normally created AUTOEXEC_Init.
 	//But if we are already running (first_shell)
-	//we have to update the envirionment to display changes
+	//we have to update the environment to display changes
 
 	if(first_shell)	{
 		//create a copy as the string will be modified
@@ -992,7 +993,7 @@ void DOS_Shell::Prepare(void) {
         initcodepagefont();
         dos.loaded_codepage=cp;
     }
-#if (defined(WIN32) && !defined(HX_DOS) || defined(LINUX) && C_X11 || defined(MACOSX) && defined(SDL_DOSBOX_X_IME)) && (defined(C_SDL2) || defined(SDL_DOSBOX_X_SPECIAL))
+#if (defined(WIN32) && !defined(HX_DOS) || defined(LINUX) && C_X11 || defined(MACOSX)) && (defined(C_SDL2) || defined(SDL_DOSBOX_X_SPECIAL))
     if (enableime) SetIMPosition();
 #endif
 }
@@ -1065,8 +1066,10 @@ void DOS_Shell::Run(void) {
 
 		/* do it */
 		if(strlen(input_line)!=0) {
+			if (bf == NULL/*not running a batch file*/ && shell_keyboard_flush) DOS_FlushSTDIN();
 			ParseLine(input_line);
 			if (echo && !bf) WriteOut_NoParsing("\n");
+			if (bf == NULL/*not running a batch file*/ && shell_keyboard_flush) DOS_FlushSTDIN();
 		}
 	} while (perm||!exit);
 	shellrun=false;
